function I = mutualinfo(X,Y,indX,indY)
%%I = mutualinfo(X,Y,N);
%I = mutualinfo(X,Y,indX,indY);
%
%mutualinfo computes the mututal information between X and Y. X and Y must
%be vectors that are the same length.
%
%Parameters
%   X           =   a vector of values for the random variable X.
%   Y           =   a vector of values for the random variable Y.
%   indX        =   the bin centers for the values of X.
%   indY        =   the bin centers for the values of Y.
%   N           =   the number of bins for the data.

if(nargin < 4)
    N = indX;
    mmX = minmax(X);
    indX = linspace(mmX(1),mmX(2),N+1);
    mmY = minmax(Y);
    indY = linspace(mmY(1),mmY(2),N+1);
end

NX = length(indX)-1;
NY = length(indY)-1;

pX = makePDF(X,indX);
pY = makePDF(Y,indY);
HX = entropy(pX);

HCXY = zeros(NY,1);
for k = 1:NY
    if(k ~= NY)
        fnd = find(Y >= indY(k) & Y < indY(k+1));
    else
        fnd = find(Y >= indY(k) & Y <= indY(k+1));
    end
    if(~isempty(fnd))
        pXCY = makePDF(X(fnd),indX);
        HCXY(k) = entropy(pXCY);
    end
end

HXCY = pY*HCXY;
I = HX - HXCY;

if(I < 0)
    ww = 0;
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function H = entropy(p)

% ENTROPY computes the entropy of a distribution.
% --------------
% H  = entropy(p)
% --------------
% Description: computes the entropy of a distribution.
% Input:       {p} is the discrete distibution. For two-element
%                   distribution, it suffices to supply with only one of
%                   the elements.
% Output:      {H} is the entropy value (in bits).

%  Liran Carmel
% Classification: Basic functions
% Last revision date: 29-Dec-2006

% check number of argumesnts
error(nargchk(1,1,nargin));

% check input
if length(p) == 1
    p = [p 1-p];
end
if abs(sum(p) - 1) > 1e-10
    error('Probability function is not normalized');
end

% calculate entropy
warning off
H = -p.*log2(p);
warning on
H = nansum(H);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [p ind] = makePDF(x,n)
%[p ind] = makePDF(x,n);
%[p ind] = makePDF(x,ind);
%
%makePDF takes data in x and makes a probability distribution with N bins.
%
%Variables
%   p           =   The probability distribution.
%   ind         =   The bin center corresponding to each distribution value.
%   x           =   Data to be binned.
%   n           =   The number of bins.

eps = 1e-3;
rng = minmax(x);
if(length(n) == 1)
    ind = linspace(rng(1),rng(2),N+1);
else
    ind = n;
end
ind(1) = ind(1)-eps;
ind(end) = ind(end)+eps;

p = histc(x,ind);
p = p(1:end-1)./sum(p);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

