function [] = makemovie(time,data,filename,framerate,qual)
%makemovie(t,data,filename,framerate)
%makemovie(t,data,filename,framerate,quality)
%
%makemovie makes a movie of 3-dimensional data. framerate is the speed
%the data is presented in the movie and quality is a compression variable
%for the movie.

if(nargin < 5)
    qual = 10;
end
try
    if(isequal(filename(end-3:end),'.avi')) 
        filename(1:end-4);
    end
end
[Ni Nj Nt] = size(data);

try
    try
        mov = avifile(filename,'Compression','Indeo5','fps',framerate);
    catch
        mov = avifile(filename,'fps',framerate);
    end
    mov.Quality = qual;

    warning off;
    cb = bounds(data,2);
    pt = time(1);
    
    h = figure; 
    setP([391 327 788 677]);
    for t = 1:length(time)
        if(mod(time(t),50) == 0)
            pt = printtime2(time(t));
        end
        imagesc(squeeze(data(:,:,t)));
        caxis(cb); colorbar; setFS(12);
        setXT; setXTL; setYT; setYTL;
        title(pt(1:end-2));

        set(gcf,'DoubleBuffer','on');
        frame{t} = getframe(gcf);
    end
    close all;
    pause(.01);
    
    %Compiling the frames.
    disp('Compiling Movie...');
    str = 'mov = addframe(mov';
    for k = 1:Nt
        str = [str ',frame{' int2str(k) '}']; 
    end
    str = [str ');'];
    warning off;
    eval(str);
    warning on;

    %Closing the movie.
    mov = close(mov);
    avireset;
catch
    disp(lasterr);
    avireset;
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = avireset(filename);
%avireset;
%avireset(filename);
%
%avireset resets the ability to generate .avi files. If a filename is 
%specified, avireset deletes the current file. 

clear mex;
if(nargin > 0)
    if(~isequal(filename(end-3:end),'.avi')) 
        filename = [filename '.avi']; 
    end
    delete(filename);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function cbounds = bounds(x, numstd);
%[mi, ma] = bounds(x, numstd);
%[mi, ma] = bounds(x);
if(nargin == 1)
    numstd = 2.5;
end
s = size(x);
len = 1;
for i = 1:length(s)
    len = len*s(i);
end
y = 1:len;
for i = 1:len
    y(i) = x(i);
end
y = y(find(y*0 == 0));
me = mean(y);
st = std(y);
mi = me - numstd*st;
ma = me + numstd*st;
cbounds = [mi ma];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function r = printtime2(printtimevar)
%str = printtime2(t)
%
%printtime2 takes a time and returns a string that can be printed.

if(printtimevar <= -1000 & printtimevar > -10000)
    title(['time:' ' ' int2str(printtimevar) ' ms']);
elseif(printtimevar <= -100 & printtimevar > -1000)
    r = ['time:' '  ' int2str(printtimevar) ' ms'];
elseif(printtimevar <= -10 & printtimevar > -100)
    r = ['time:' '   ' int2str(printtimevar) ' ms'];
elseif(printtimevar < 0 & printtimevar > -10)
    r = ['time:' '    ' int2str(printtimevar) ' ms'];
elseif(printtimevar < 10 & printtimevar >= 0)
    r = ['time:' '    ' int2str(printtimevar) ' ms'];
elseif(printtimevar < 100 & printtimevar >= 10)
    r = ['time:' '   ' int2str(printtimevar) ' ms'];
elseif(printtimevar < 1000 & printtimevar >= 100)
    r = ['time:' '  ' int2str(printtimevar) ' ms'];
elseif(printtimevar < 10000 & printtimevar >= 1000)
    r = ['time:' ' ' int2str(printtimevar) ' ms'];
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%